--[[=============================================================================
    Alarm/Trouble events and notification handling

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]
require "lib.c4_log"
require "panel_strings"
require "deocean_callbacks"
require "panel_proxy.securitypanel_functions"

function ProcessEvent(partitionID, sourceType, sourceNumber, generalType, specificType, eventDataHigh, eventDataLow)

	LogDebug("ProcessEvent(): GeneralType[%d]    SpecificType[%d]", generalType, specificType)
	if (generalType >= 1 and generalType <= 3) then
		Alarm(partitionID, generalType, specificType, eventDataHigh, eventDataLow)
	elseif (generalType >= 4 and generalType <= 7) then
		Trouble(partitionID, sourceType, sourceNumber, generalType, specificType)
	elseif (generalType == 8 or generalType == 9) then
		Bypass(partitionID, generalType, specificType, eventDataHigh, eventDataLow)
	elseif (generalType == 10) then
		Open(partitionID, specificType, eventDataHigh, eventDataLow)
	elseif (generalType == 11) then
		Close(partitionID, specificType, eventDataHigh, eventDataLow)
	elseif (generalType == 12) then
		PartitionConfig(partitionID, specificType, eventDataHigh, eventDataLow)
	elseif (generalType == 13) then
		PartitionEvent(partitionID, specificType, eventDataHigh, eventDataLow)
	elseif (generalType == 14) then
		PartitionTest(partitionID, specificType, eventDataHigh, eventDataLow)
	elseif (generalType == 15 or generalType == 16) then
		SystemTrouble(partitionID, generalType, specificType, eventDataHigh, eventDataLow)
	elseif (generalType == 17) then
		SystemConfig(partitionID, specificType)
	elseif (generalType == 18) then
		SystemEvent(partitionID, specificType, eventDataHigh, eventDataLow)
	end
end

function Alarm(partitionID, generalType, specificType, eventDataHigh, eventDataLow)
	local user = ""
	local eventData = ""
	local message = AlarmValues[tostring(specificType)] or ""

	if (generalType == 2) then
		message = message .. " " .. canceled
	elseif (generalType == 3) then
		message = message .. " " .. restored
	end

	if (generalType == 1) then

		if (specificType == 21) then
			eventData = latchkey
		end

		SetPartitionState(tonumber(partitionID), "ALARM", message, 0, 0, true)
	else

		if (specificType == 21) then
			eventData = latchkey .. " " .. canceled
		end
	end

	LogDebug("Alarm(%s)", message)
	if (specificType == 21) then
		user = GetUserNumber(eventDataHigh, eventDataLow)
		UpdateC4Variable(partitionID, user, eventData)
	end
end

function Trouble(partitionID, sourceType, sourceNumber, generalType, specificType)
	local message = TroubleValues[tostring(specificType)] or ""
	local eventData = SourceType[tostring(sourceType)] .. " " .. tostring(sourceNumber) .. " - " .. TroubleValues[tostring(specificType)]

	LogDebug("Trouble(%s)", message)
	if (generalType == 4 or generalType == 6) then
		-- Start Trouble
		StartTroubleCondition(message)
	else
		-- End Trouble
		ClearTroubleCondition(message)
		eventData = eventData .. " " .. restored
	end

	UpdateC4Variable(partitionID, "", eventData)
end

function Bypass(partitionID, generalType, specificType, eventDataHigh, eventDataLow)
	local user = ""
	local message
	
	if (generalType == 8) then 
		message = BypassValues[tostring(specificType)] or ""
		LogDebug("Bypass(%s)", message)
	else
		message = UnbypassValues[tostring(specificType)] or ""
		LogDebug("Unbypass(%s)", message)
	end

	DisplayPartitionText(partitionID, message)
	if (specificType < 2) then
		user = GetUserNumber(eventDataHigh, eventDataLow)
	end

	UpdateC4Variable(partitionID, user, message)
end

function Open(partitionID, specificType, eventDataHigh, eventDataLow)
	local user = ""
	local message = OpeningValues[tostring(specificType)] or ""

	LogDebug("Open(%s)", message)
	DisplayPartitionText(partitionID, message)

	if (specificType == 1 or specificType == 2) then
		user = GetUserNumber(eventDataHigh, eventDataLow)
	end

	UpdateC4Variable(partitionID, user, message)
end

function Close(partitionID, specificType, eventDataHigh, eventDataLow)
	local user = ""
	local message = ClosingValues[tostring(specificType)] or ""

	LogDebug("Close(%s)", message)
	DisplayPartitionText(partitionID, message)

	if (specificType == 1 or specificType == 2 or specificType == 9) then
		user = GetUserNumber(eventDataHigh, eventDataLow)
	end

	UpdateC4Variable(partitionID, user, message)
end

function PartitionConfig(partitionID, specificType, eventDataHigh, eventDataLow)
	local message = PartitionConfigValues[tostring(specificType)] or ""

	LogDebug("PartitionConfig(%s)", message)
	DisplayPartitionText(partitionID, message)
end

function PartitionEvent(partitionID, specificType, eventDataHigh, eventDataLow)
	local message = PartitionEventValues[tostring(specificType)] or ""

	LogDebug("PartitionConfig(%s)", message)
	DisplayPartitionText(partitionID, message)

	if (specificType < 2) then
		UpdateC4Variable(partitionID, "", message .. "[" .. tostring(eventDataLow) .. "]")
	else
		if (specificType == 12 or specificType >= 15) then
			UpdateC4Variable(partitionID, GetUserNumber(eventDataHigh, eventDataLow), message)
		end
	end

	if (specificType == 15) then
		-- Arming Protest Beginning
		ArmPartitionFailed(partitionID, "Bypass", LAST_INTERFACEID)
	else
		if (specificType == 16) then
			-- Arming Protest Complete
			UpdatePartitionStatus(partitionID, tonumber(eventDataLow))
		end
	end
end

function PartitionTest(partitionID, specificType, eventDataHigh, eventDataLow)
	local user = ""
	local message = PartitionTestValues[tostring(specificType)] or ""

	LogDebug("PartitionTest(%s)", message)
	DisplayPartitionText(partitionID, message)

	if (specificType == 0 or specificType == 5 or specificType == 6) then
		user = GetUserNumber(eventDataHigh, eventDataLow)
	end

	UpdateC4Variable(partitionID, user, message)
end

function SystemTrouble(partitionID, generalType, specificType, eventDataHigh, eventDataLow)
	local message = SystemTroubleValues[tostring(specificType)] or ""

	if (generalType == 15) then
		-- Start Trouble
		StartTroubleCondition(message)
	else
		-- End Trouble
		ClearTroubleCondition(message)
		message = message .. " " .. restored
	end

	LogDebug("SystemTrouble(%s)", message)
	DisplayPartitionText(partitionID, message)

	UpdateC4Variable(partitionID, "", message)
end

function SystemConfig(partitionID, specificType)
	local message = SystemConfigValues[tostring(specificType)] or ""

	LogDebug("SystemConfig(%s)", message)
	DisplayPartitionText(partitionID, message)

	if (specificType == 0) then
		-- Programming Mode Entry
		ProgrammingMode = true
	else
		if (specificType == 1 or specificType == 2) then
			-- Programming Mode Exit
			ProgrammingMode = false
		end
	end
end

function SystemEvent(partitionID, specificType, eventDataHigh, eventDataLow)
	local message = SystemEventValues[tostring(specificType)] or ""
	local pgmNumber = tonumber(eventDataLow)
	local user = GetUserNumber(eventDataHigh, eventDataLow);

	LogDebug("SystemEvent(%s)", message)
	DisplayPartitionText(partitionID, message)

	if (specificType == 5 or specificType == 6) then
		local isControllable = Properties["Output " .. pgmNumber .. " Controllable"]

		UpdateC4Variable(partitionID, user, message)
		if (toboolean(isControllable)) then
			if (IsPgmValid(pgmNumber) == false) then
				AddPgm(pgmNumber)
			end

			local pgmState = false
			if (specificType == 5) then  -- Default is "off" so change it to "on" when type is 5
				pgmState = true
			end

			SetPgmState(pgmNumber, pgmState, false)
		end
	end
end