--[[=============================================================================
    Functions for handling and executing commands and actions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]
require "common.c4_driver_declarations"
require "device_messages"

-- Set template version for this file
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.c4_command = "2015.03.02"
end

--[[=============================================================================
    ExecuteCommand(sCommand, tParams)

    Description
    Function called by Director when a command is received for this DriverWorks
    driver. This includes commands created in Composer programming.

    Parameters
    sCommand(string) - Command to be sent
    tParams(table)   - Lua table of parameters for the sent command

    Returns
    Nothing
===============================================================================]]
function ExecuteCommand(sCommand, tParams)
	LogTrace("ExecuteCommand(" .. sCommand .. ")")
	LogInfo(tParams)

	-- Remove any spaces (trim the command)
	local trimmedCommand = string.gsub(sCommand, " ", "")
	local status, err

	-- if function exists then execute (non-stripped)
	if (EX_CMD[sCommand] ~= nil and type(EX_CMD[sCommand]) == "function") then
		status, err = pcall(EX_CMD[sCommand], tParams)
	-- elseif trimmed function exists then execute
	elseif (EX_CMD[trimmedCommand] ~= nil and type(EX_CMD[trimmedCommand]) == "function") then
		status, err = pcall(EX_CMD[trimmedCommand], tParams)
	elseif (EX_CMD[sCommand] ~= nil) then
		QueueCommand(EX_CMD[sCommand])
		status = true
	else
		LogInfo("ExecuteCommand: Unhandled command = " .. sCommand)
		status = true
	end
	
	if (not status) then
		LogError("LUA_ERROR: " .. err)
	end
end

--[[=============================================================================
    EX_CMD.LUA_ACTION(tParams)

    Description
    Function called for any actions executed by the user from the Actions Tab
    in Composer.

    Parameters
    tParams(table) - Lua table of parameters for the command option

    Returns
    Nothing
===============================================================================]]
function EX_CMD.LUA_ACTION(tParams)

	if (tParams ~= nil) then
		for cmd, cmdv in pairs(tParams) do
			if (cmd == "ACTION" and cmdv ~= nil) then
				local status, err = pcall(LUA_ACTION[cmdv], tParams)
				if (not status) then
					LogError("LUA_ERROR: " .. err)
				end

				break
			else
				LogInfo("Undefined Command")
				LogInfo("Key: " .. cmd .. " Value: " .. cmdv)
			end
		end
	end
end

--[[=============================================================================
    ReceivedFromProxy(idBinding, sCommand, tParams)

    Description
    Function called for any actions executed by the user from the Actions Tab
    in Composer.

    Parameters
    idBinding(int)   - Binding ID of the proxy that sent a BindMessage to the
                       DriverWorks driver.
    sCommand(string) - Command that was sent
    tParams(table)   - Lua table of received command parameters

    Returns
    Nothing
===============================================================================]]
function ReceivedFromProxy(idBinding, sCommand, tParams)

	if (sCommand ~= nil) then

		-- initial table variable if nil
		if (tParams == nil) then
			tParams = {}
		end
		
		LogTrace("ReceivedFromProxy(): " .. sCommand .. " on binding " .. idBinding .. "; Call Function PRX_CMD." .. sCommand .. "()")
		LogInfo(tParams)
				
		     --[[if(sCommand == "BusOut" ) then --IO_DATA_RECEIVE_FROM_50005
				--Process the data from 50005
				--print_recv_data(tParams["pDeviceID"], tParams["pCommand"], tParams["data"])
				local pDeviceID	= tonumber("0x" .. string.format("%02X",tParams["C"]),10)
				local pCommand 	= tParams["B"]
				local data 		= string.format("%02d%02d", tParams["D"], tParams["E"])
				ParseRecvDataFrom50005(pDeviceID, pCommand, data)
			end]]
			
				
	     --self:ReceivedFromCom(sData)
		if ((PRX_CMD[sCommand]) ~= nil) then
			local status, err = pcall(PRX_CMD[sCommand], idBinding, tParams)
			if (not status) then
				LogError("LUA_ERROR: " .. err)
			end
		else
			LogInfo("ReceivedFromProxy: Unhandled command = " .. sCommand)
		end
	end
end



--[[FromSerialCommand = "ZoneData"
FromSerialData = "0102000000000131"    
DispatchMessage(FromSerialCommand, FromSerialData)]]



--[[function ParseRecvDataFrom50005(deviceID, command, data)
		  
     LogTrace("ReceivedFromProxy cmd: pDeviceID , cmdv: " .. tonumber(deviceID))
     LogTrace("ReceivedFromProxy cmd: pCommand , cmdv: " .. tonumber(command))
     LogTrace("ReceivedFromProxy cmd: data , cmdv: " .. data)
		  
	if(tonumber(deviceID) ~= tonumber("0x" .. Properties["Device ID"],10)) or (tonumber(command) ~= 255)then
	   return
	end
		  
	--Analysis data
	--data index from 1 to n
     local num 	= string.sub(data,1,2)
     local fault	= string.sub(data,3,4)
		  
     ProcessMessage(tonumber(num),tonumber(fault))
     SendReturnData(idBinding,tonumber(deviceID),tonumber(num),tonumber(fault))
     --HandleMessage(data)
end

function SendReturnData(idBinding,deviceID,num,fault)
	if (idBinding == 1) then
		local Command = hexmap(deviceID) .. " " .. hexmap(num) .. " " .. hexmap(fault)
		C4:SendToProxy(idBinding,Command,{})
	end
end

function hexmap(numb)
	local hexnum,highbite
	if (numb > 9 ) then
		if (numb == 10) then
			hexnum = 'A' 
			highbite = '0'
		elseif (numb == 11) then
			hexnum = 'B'
			highbite = '0'
		elseif (numb == 12) then
			hexnum = 'C'
			highbite = '0'
		elseif (numb == 13) then
			hexnum = 'D'
			highbite = '0'
		elseif (numb == 14) then
			hexnum = 'E'
			highbite = '0'
		elseif (numb == 15) then
			hexnum = 'F'
			highbite = '0'
		elseif (numb > 15) then
			hexnum = (numb -16)
			highbite = '1'
		else
		end
	else
		hexnum = numb
		highbite = '0'
	end
	return highbite .. hexnum
end

function ProcessMessage(num,fault)
     
	if(fault == 1) then
		fault = "Close"
		if (num == 1) then
			LogTrace("Sensor 1  ,Status: " .. fault)
			local eventName = "Sensor 1 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(101, "CLOSED", {}, "NOTIFY")
		elseif (num == 2) then
			LogTrace("Sensor 2  ,Status: " .. fault)
			local eventName = "Sensor 2 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(102, "CLOSED", {}, "NOTIFY")
		elseif (num == 3) then
			LogTrace("Sensor 3  ,Status: " .. fault)
			local eventName = "Sensor 3 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(103, "CLOSED", {}, "NOTIFY")
		elseif (num == 4) then
			LogTrace("Sensor 4  ,Status: " .. fault)
			local eventName = "Sensor 4 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(104, "CLOSED", {}, "NOTIFY")
		elseif (num == 5) then
			LogTrace("Sensor 5  ,Status: " .. fault)
			local eventName = "Sensor 5 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(105, "CLOSED", {}, "NOTIFY")
		elseif (num == 6) then
			LogTrace("Sensor 6  ,Status: " .. fault)
			local eventName = "Sensor 6 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(106, "CLOSED", {}, "NOTIFY")
		elseif (num == 7) then
			LogTrace("Sensor 7  ,Status: " .. fault)
			local eventName = "Sensor 7 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(107, "CLOSED", {}, "NOTIFY")
		elseif (num == 8) then
			LogTrace("Sensor 8  ,Status: " .. fault)
			local eventName = "Sensor 8 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(108, "CLOSED", {}, "NOTIFY")
		elseif (num == 9) then
			LogTrace("Sensor 9  ,Status: " .. fault)
			local eventName = "Sensor 9 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(109, "CLOSED", {}, "NOTIFY")
		elseif (num == 10) then
			LogTrace("Sensor 10  ,Status: " .. fault)
			local eventName = "Sensor 10 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(110, "CLOSED", {}, "NOTIFY")
		elseif (num == 11) then
			LogTrace("Sensor 11  ,Status: " .. fault)
			local eventName = "Sensor 11 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(111, "CLOSED", {}, "NOTIFY")
		elseif (num == 12) then
			LogTrace("Sensor 12  ,Status: " .. fault)
			local eventName = "Sensor 12 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(112, "CLOSED", {}, "NOTIFY")
		elseif (num == 13) then
			LogTrace("Sensor 13  ,Status: " .. fault)
			local eventName = "Sensor 13 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(113, "CLOSED", {}, "NOTIFY")
		elseif (num == 14) then
			LogTrace("Sensor 14  ,Status: " .. fault)
			local eventName = "Sensor 14 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(114, "CLOSED", {}, "NOTIFY")
		elseif (num == 15) then
			LogTrace("Sensor 15  ,Status: " .. fault)
			local eventName = "Sensor 15 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(115, "CLOSED", {}, "NOTIFY")
		elseif (num == 16) then
			LogTrace("Sensor 16  ,Status: " .. fault)
			local eventName = "Sensor 16 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(116, "CLOSED", {}, "NOTIFY")
		elseif (num == 17) then
			LogTrace("Sensor 17  ,Status: " .. fault)
			local eventName = "Sensor 17 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(117, "CLOSED", {}, "NOTIFY")
		elseif (num == 18) then
			LogTrace("Sensor 18  ,Status: " .. fault)
			local eventName = "Sensor 18 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(118, "CLOSED", {}, "NOTIFY")
		elseif (num == 19) then
			LogTrace("Sensor 19  ,Status: " .. fault)
			local eventName = "Sensor 19 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(119, "CLOSED", {}, "NOTIFY")
		elseif (num == 20) then
			LogTrace("Sensor 20  ,Status: " .. fault)
			local eventName = "Sensor 20 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(120, "CLOSED", {}, "NOTIFY")
		elseif (num == 21) then
			LogTrace("Sensor 21  ,Status: " .. fault)
			local eventName = "Sensor 21 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(121, "CLOSED", {}, "NOTIFY")
		elseif (num == 22) then
			LogTrace("Sensor 22  ,Status: " .. fault)
			local eventName = "Sensor 22 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(122, "CLOSED", {}, "NOTIFY")
		elseif (num == 23) then
			LogTrace("Sensor 23  ,Status: " .. fault)
			local eventName = "Sensor 23 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(123, "CLOSED", {}, "NOTIFY")
		elseif (num == 24) then
			LogTrace("Sensor 24  ,Status: " .. fault)
			local eventName = "Sensor 24 Close"
			C4:FireEvent(eventName)
			C4:SendToProxy(124, "CLOSED", {}, "NOTIFY")
		else 
			LogTrace("Undifned Commmand")
		end
     elseif (fault == 2) then
		fault = "Open"
		if (num == 1) then
			LogTrace("Sensor 1  ,Status: " .. fault)
			local eventName = "Sensor 1 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(101, "OPENED", {}, "NOTIFY")
		elseif (num == 2) then
			LogTrace("Sensor 2  ,Status: " .. fault)
			local eventName = "Sensor 2 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(102, "OPENED", {}, "NOTIFY")
		elseif (num == 3) then
			LogTrace("Sensor 3  ,Status: " .. fault)
			local eventName = "Sensor 3 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(103, "OPENED", {}, "NOTIFY")
		elseif (num == 4) then
			LogTrace("Sensor 4  ,Status: " .. fault)
			local eventName = "Sensor 4 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(104, "OPENED", {}, "NOTIFY")
		elseif (num == 5) then
			LogTrace("Sensor 5  ,Status: " .. fault)
			local eventName = "Sensor 5 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(105, "OPENED", {}, "NOTIFY")
		elseif (num == 6) then
			LogTrace("Sensor 6  ,Status: " .. fault)
			local eventName = "Sensor 6 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(106, "OPENED", {}, "NOTIFY")
		elseif (num == 7) then
			LogTrace("Sensor 7  ,Status: " .. fault)
			local eventName = "Sensor 7 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(107, "OPENED", {}, "NOTIFY")
		elseif (num == 8) then
			LogTrace("Sensor 8  ,Status: " .. fault)
			local eventName = "Sensor 8 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(108, "OPENED", {}, "NOTIFY")
		elseif (num == 9) then
			LogTrace("Sensor 9  ,Status: " .. fault)
			local eventName = "Sensor 9 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(109, "OPENED", {}, "NOTIFY")
		elseif (num == 10) then
			LogTrace("Sensor 10  ,Status: " .. fault)
			local eventName = "Sensor 10 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(110, "OPENED", {}, "NOTIFY")
		elseif (num == 11) then
			LogTrace("Sensor 11  ,Status: " .. fault)
			local eventName = "Sensor 11 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(111, "OPENED", {}, "NOTIFY")
		elseif (num == 12) then
			LogTrace("Sensor 12  ,Status: " .. fault)
			local eventName = "Sensor 12 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(112, "OPENED", {}, "NOTIFY")
		elseif (num == 13) then
			LogTrace("Sensor 13  ,Status: " .. fault)
			local eventName = "Sensor 13 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(113, "OPENED", {}, "NOTIFY")
		elseif (num == 14) then
			LogTrace("Sensor 14  ,Status: " .. fault)
			local eventName = "Sensor 14 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(114, "OPENED", {}, "NOTIFY")
		elseif (num == 15) then
			LogTrace("Sensor 15  ,Status: " .. fault)
			local eventName = "Sensor 15 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(115, "OPENED", {}, "NOTIFY")
		elseif (num == 16) then
			LogTrace("Sensor 16  ,Status: " .. fault)
			local eventName = "Sensor 16 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(116, "OPENED", {}, "NOTIFY")
		elseif (num == 17) then
			LogTrace("Sensor 17  ,Status: " .. fault)
			local eventName = "Sensor 17 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(117, "OPENED", {}, "NOTIFY")
		elseif (num == 18) then
			LogTrace("Sensor 18  ,Status: " .. fault)
			local eventName = "Sensor 18 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(118, "OPENED", {}, "NOTIFY")
		elseif (num == 19) then
			LogTrace("Sensor 19  ,Status: " .. fault)
			local eventName = "Sensor 19 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(119, "OPENED", {}, "NOTIFY")
		elseif (num == 20) then
			LogTrace("Sensor 20  ,Status: " .. fault)
			local eventName = "Sensor 20 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(120, "OPENED", {}, "NOTIFY")
		elseif (num == 21) then
			LogTrace("Sensor 21  ,Status: " .. fault)
			local eventName = "Sensor 21 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(121, "OPENED", {}, "NOTIFY")
		elseif (num == 22) then
			LogTrace("Sensor 22  ,Status: " .. fault)
			local eventName = "Sensor 22 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(122, "OPENED", {}, "NOTIFY")
		elseif (num == 23) then
			LogTrace("Sensor 23  ,Status: " .. fault)
			local eventName = "Sensor 23 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(123, "OPENED", {}, "NOTIFY")
		elseif (num == 24) then
			LogTrace("Sensor 24  ,Status: " .. fault)
			local eventName = "Sensor 24 Open"
			C4:FireEvent(eventName)
			C4:SendToProxy(124, "OPENED", {}, "NOTIFY")
		else 
			LogTrace("Undifned Commmand")
		end
	end
end]]
