--[[=============================================================================
    File for implementing driver specific functions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]
require "partition_proxy.securitypartition"

SecurityPartitionFunctions = {}

function SecurityPartitionFunctions.QuickExit(PartitionID, DeviceID)
	local lastIndex = PadValue(ConvertToHex(5))
	local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "0") .. "33"
	local checkSum = GetCheckSum(request)

	LogDebug("Function(QuickExit): request[%s]", request .. checkSum)
	SendCommand("\n" .. request .. checkSum)
end

function SecurityPartitionFunctions.Status(PartitionID, DeviceID)
	local lastIndex = PadValue(ConvertToHex(5))
	local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "0A")
	local checkSum = GetCheckSum(request)

	LogDebug("Function(Status): request[%s]", request .. checkSum)
	SendCommand("\n" .. request .. checkSum)
end

function SecurityPartitionFunctions.Chime(PartitionID, DeviceID)
	local lastIndex = PadValue(ConvertToHex(6))
	local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "071")
	local checkSum = GetCheckSum(request)

	LogDebug("Function(Chime): request[%s]", request .. checkSum)
	SendCommand("\n" .. request .. checkSum)
end

function SecurityPartitionFunctions.EnergySaver(PartitionID, DeviceID)
	local lastIndex = PadValue(ConvertToHex(6))
	local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "072")
	local checkSum = GetCheckSum(request)

	LogDebug("Functions(EnergySaver): request[%s]", request .. checkSum)
	SendCommand("\n" .. request .. checkSum)
end

function SecurityPartitionFunctions.AlarmInMemory(PartitionID, DeviceID)
	local lastIndex = PadValue(ConvertToHex(6))
	local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "076")
	local checkSum = GetCheckSum(request)

	LogDebug("Functions(AlarmInMemory): request[%s]", request .. checkSum)
	SendCommand("\n" .. request .. checkSum)
end

function SecurityPartitionFunctions.BypassSensor(PartitionID, DeviceID, UserCode, Sensor)
	local Parameters

	if (UserCode == nil) then
		Parameters = tostring(PartitionID) .. "," .. tostring(DeviceID)
		LogDebug("BypassSensor Parameters(Get Usercode): %s", Parameters)
		PartitionRequestAdditionalInfo(PartitionID, GetText("604D"), Parameters, "SecurityPartitionFunctions.BypassSensor", true, DeviceID)
	else
		if (Sensor == nil) then
			Parameters = tostring(PartitionID) .. "," .. tostring(DeviceID) .. "," .. tostring(UserCode)
			LogDebug("BypassSensor Parameters(Get Sensor): %s", tostring(Parameters))
			PartitionRequestAdditionalInfo(PartitionID, GetText("B396"), tostring(Parameters), "SecurityPartitionFunctions.BypassSensor", false, DeviceID)
		else
			parms = {}
			parms["SensorID"] = Sensor
			parms["PartitionID"] = PartitionID

			BypassSensorTimer = CreateTimer("Zone Bypass", 500, "MILLISECONDS", BypassSensorCB, false, parms)
			StartTimer(BypassSensorTimer)

			local lastIndex = PadValue(ConvertToHex(5+string.len(UserCode)))
			local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "0B") .. PadKeyPresses(UserCode)
			local checkSum = GetCheckSum(request)

			LogDebug("Function(BypassSensor): request[%s]", request .. checkSum)
			SendCommand("\n" .. request .. checkSum)
		end
	end
end

function SecurityPartitionFunctions.AddUserCode(PartitionID, DeviceID, InstallerCode, UserNumber, UserCode, VerifiedCode)
	local Parameters = tostring(PartitionID) .. "," .. tostring(DeviceID)

	if (InstallerCode == nil) then
		LogDebug("AddUserCode Parameters(Get Installer Code): %s", Parameters)
		PartitionRequestAdditionalInfo(PartitionID, GetText("604D"), Parameters, "SecurityPartitionFunctions.AddUserCode", true, DeviceID)
	else
		if (UserNumber == nil) then
			Parameters = Parameters .. "," .. tostring(InstallerCode)
			LogDebug("AddUserCode Parameters(Get UserNumber): %s", tostring(Parameters))
			PartitionRequestAdditionalInfo(PartitionID, GetText("60E096"), tostring(Parameters), "SecurityPartitionFunctions.AddUserCode", false, DeviceID)
		else
			if (UserCode == nil) then
				Parameters = Parameters .. "," .. tostring(InstallerCode) .. "," .. tostring(UserNumber)
				LogDebug("AddUserCode Parameters(Get UserCode): %s", tostring(Parameters))
				PartitionRequestAdditionalInfo(PartitionID, GetText("60E14D"), tostring(Parameters), "SecurityPartitionFunctions.AddUserCode", true, DeviceID)
			else
				if (VerifiedCode == nil) then
					Parameters = Parameters .. "," .. tostring(InstallerCode) .. "," .. tostring(UserNumber) .. "," .. tostring(UserCode)
					LogDebug("AddUserCode Parameters(Re-enter New UserCode): %s", tostring(Parameters))
					PartitionRequestAdditionalInfo(PartitionID, GetText("E3E04D"), tostring(Parameters), "SecurityPartitionFunctions.AddUserCode", true, DeviceID)
				else
					if (UserCode ~= VerifiedCode) then
						Parameters = Parameters .. "," .. tostring(InstallerCode) .. "," .. tostring(UserNumber)
						LogDebug("AddUserCode Parameters(Codes Don't Match): %s", tostring(Parameters))
						PartitionRequestAdditionalInfo(PartitionID, GetText("E4E5FB60E2E04D"), tostring(Parameters), "SecurityPartitionFunctions.AddUserCode", true, DeviceID)
					else
						parms = {}
						parms["UserNumber"] = UserNumber
						parms["UserCode"] = UserCode

						ManageUserCodeTimer = CreateTimer("Add User Code", 500, "MILLISECONDS", ManageUserCodeCB, true, parms)
						StartTimer(ManageUserCodeTimer)

						local lastIndex = PadValue(ConvertToHex(5 + string.len(InstallerCode)))
						local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "09" .. InstallerCode)
						local checkSum = GetCheckSum(request)

						LogDebug("Function(AddUserCode): request[%s]", request .. checkSum)
						SendCommand("\n" .. request .. checkSum)
					end
				end
			end
		end
	end
end

function SecurityPartitionFunctions.ChangeUserCode(PartitionID, DeviceID, InstallerCode, UserNumber, UserCode, VerifiedCode)
	local Parameters = tostring(PartitionID) .. "," .. tostring(DeviceID)

	if (InstallerCode == nil) then
		LogDebug("ChangeUserCode Parameters(Get Installer Code): %s", Parameters)
		PartitionRequestAdditionalInfo(PartitionID, GetText("604D"), Parameters, "SecurityPartitionFunctions.ChangeUserCode", true, DeviceID)
	else
		if (UserNumber == nil) then
			Parameters = Parameters .. "," .. tostring(InstallerCode)
			LogDebug("ChangeUserCode Parameters(Get UserNumber): %s", tostring(Parameters))
			PartitionRequestAdditionalInfo(PartitionID, GetText("60E096"), tostring(Parameters), "SecurityPartitionFunctions.ChangeUserCode", false, DeviceID)
		else
			if (UserCode == nil) then
				Parameters = Parameters .. "," .. tostring(InstallerCode) .. "," .. tostring(UserNumber)
				LogDebug("ChangeUserCode Parameters(Get New UserCode): %s", tostring(Parameters))
				PartitionRequestAdditionalInfo(PartitionID, GetText("60E2E04D"), tostring(Parameters), "SecurityPartitionFunctions.ChangeUserCode", true, DeviceID)
			else
				if (VerifiedCode == nil) then
					Parameters = Parameters .. "," .. tostring(InstallerCode) .. "," .. tostring(UserNumber) .. "," .. tostring(UserCode)
					LogDebug("ChangeUserCode Parameters(Re-enter New UserCode): %s", tostring(Parameters))
					PartitionRequestAdditionalInfo(PartitionID, GetText("E3E2E04D"), tostring(Parameters), "SecurityPartitionFunctions.ChangeUserCode", true, DeviceID)
				else
					if (UserCode ~= VerifiedCode) then
						Parameters = Parameters .. "," .. tostring(InstallerCode) .. "," .. tostring(UserNumber)
						LogDebug("ChangeUserCode Parameters(Codes Don't Match): %s", tostring(Parameters))
						PartitionRequestAdditionalInfo(PartitionID, GetText("E4E5FB60E2E04D"), tostring(Parameters), "SecurityPartitionFunctions.ChangeUserCode", true, DeviceID)
					else
						parms = {}
						parms["UserNumber"] = UserNumber
						parms["UserCode"] = UserCode

						ManageUserCodeTimer = CreateTimer("Change User Code", 500, "MILLISECONDS", ManageUserCodeCB, true, parms)
						StartTimer(ManageUserCodeTimer)

						local lastIndex = PadValue(ConvertToHex(5 + string.len(InstallerCode)))
						local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "09" .. InstallerCode)
						local checkSum = GetCheckSum(request)

						LogDebug("Function(ChangeUserCode): request[%s]", request .. checkSum)
						SendCommand("\n" .. request .. checkSum)
					end
				end
			end
		end
	end
end

function SecurityPartitionFunctions.DeleteUserCode(PartitionID, DeviceID, InstallerCode, UserNumber)
	local Parameters = tostring(PartitionID) .. "," .. tostring(DeviceID)

	if (InstallerCode == nil) then
		LogDebug("DeleteUserCode Parameters(Get Installer Code): %s", Parameters)
		PartitionRequestAdditionalInfo(PartitionID, GetText("604D"), Parameters, "SecurityPartitionFunctions.DeleteUserCode", true, DeviceID)
	else
		if (UserNumber == nil) then
			Parameters = Parameters .. "," .. tostring(InstallerCode)
			LogDebug("DeleteUserCode Parameters(Get UserNumber): %s", tostring(Parameters))
			PartitionRequestAdditionalInfo(PartitionID, GetText("60E096"), tostring(Parameters), "SecurityPartitionFunctions.DeleteUserCode", false, DeviceID)
		else
			parms = {}
			parms["UserNumber"] = UserNumber
			parms["UserCode"] = InstallerCode

			ManageUserCodeTimer = CreateTimer("Delete User Code", 500, "MILLISECONDS", ManageUserCodeCB, true, parms)
			StartTimer(ManageUserCodeTimer)

			local lastIndex = PadValue(ConvertToHex(5 + string.len(InstallerCode)))
			local request = lastIndex .. "40" .. PadKeyPresses(PartitionID .. "09" .. InstallerCode)
			local checkSum = GetCheckSum(request)

			LogDebug("Function(DeleteUserCode): request[%s]", request .. checkSum)
			SendCommand("\n" .. request .. checkSum)
		end
	end
end

function SecurityPartitionFunctions.ClearBufferMemory(PartitionID, DeviceID, InstallerCode)
end
